assert(TrackCooldowns, "TrackCooldowns not found!")

local MINOR_VERSION = tonumber(("$Revision: 2 $"):match("%d+"))
if MINOR_VERSION > TrackCooldowns.MINOR_VERSION then TrackCooldowns.MINOR_VERSION = MINOR_VERSION end

local pairs = _G.pairs
local GetTime = _G.GetTime
local GetSpellCooldown = _G.GetSpellCooldown

local base = {}

function base:OnInitialize()
	self:SetEnabledState(false)
end

function base:OnEnable()
	self.canScanCooldowns = true
	self:ScanSpells()
	self:RegisterEvent("CHARACTER_POINTS_CHANGED", "Respec")
	self:RegisterEvent("CONFIRM_TALENT_WIPE", "Respec")
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "Respec")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "GenericSpellSuccess")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "ScanSpells")
end

function base:OnDisable()
	self:UnregisterAllEvents()
end

function base:Super(t)
	local sup = getmetatable(self)["__index"]
	return sup[t](sup == base and self or sup)
end

--------------[[		Combat Events		]]--------------

function base:GenericSpellSuccess(event, unit, spell)
	if unit ~= "player" then return end
	if self.cooldowns[spell] == nil then return end
	self.canScanCooldowns = true
end

function base:Respec(event)
	self.canScanCooldowns = true
	self:ScanSpells()
end

function base:GetCooldown(spell)
	local remaining = 0
	local startTime, cooldown, enabled = GetSpellCooldown(spell)

	if startTime and startTime > 0 and cooldown >= 2 and enabled == 1 then
		remaining = math.ceil(startTime + cooldown - GetTime())
	end
	
	return remaining
end

--------------[[		Comm Methods		]]--------------

function base:Sync(data, cooldown)
	if cooldown == nil then return end
	
	local id = data
	if type(data) == "table" then
		id = data.id
		
		if not oRA and data.ora ~= nil then
			TrackCooldowns:SendCommMessage("oRA", "CD " .. data.ora .. " " .. (cooldown / 60), "RAID")
		end
	end
	
	TrackCooldowns:SendCommMessage(TrackCooldowns.prefix, (id .. " " .. cooldown), "RAID")
end

function base:ScanSpells()
	if self.canScanCooldowns == false then return end
	
	local cooldown
	for k, v in pairs(self.cooldowns) do
		cooldown = self:GetCooldown(k)
		if cooldown == 0 or cooldown > 2 then
			self:Sync(v, cooldown)
		end
	end
	
	self.canScanCooldowns = false
end

TrackCooldowns.ModuleBase = base